//**********************************************
//
//  MakeVsta
//     Ver. 20111019
//        T.Sako
//
//**********************************************
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <vector>
#include <map>

static std::string QDCModel;

void ExitFunction(int SFflag)
{
  using namespace std;
  if(SFflag == 1){
    cout << "    === Conversion is completed !! === " << endl ; 
  }
  
  cout << "-------------------------------" << endl; 
  exit(0);
}

void OLine(std::string Line){
	using namespace std;
	cout << Line << endl;
}

void DisplayHelp(){
	using namespace std;

	OLine("    ");
	OLine("Usage : MakeVsta peakfile sigmafile outputfile mapfile [-d]");
	OLine("");
	OLine(" -d : detail mode. Show detail of conversion and generate the .chk file.");
	OLine("      .chk shows GEO, Channel and detector name.");
	OLine("");
	OLine(" MakeVsta generates .vsta using patameter file and map file.");
	OLine(" More details : http://be.nucl.ap.titech.ac.jp/~sako/tool/MakeVsta.html");
	OLine("");
	OLine("        MakeVsta");
	OLine("          Ver. 20111019");
	OLine("          by T.Sako");
	OLine("");
	exit(0);
}


int GEOIDGen(int GEO, int ID){
  int GEOID;
  GEOID = GEO * 100 + ID;
  return GEOID;
}

int ConvertFunction(float ImportPeak, float ImportSigma, float CutMargin,  int forbidden)
{
  float Output;

  if ((int)ImportPeak == 0){
    Output = forbidden;
  }else{
    Output = (ImportPeak + ImportSigma * CutMargin)/2.;
  }
  return (int)Output;
}


std::string IniCommentString(std::vector<std::string>& LineVector, int& LineNo){
  using namespace std;

  string CommentString = "#";
  string InitString;
  
  InitString = LineVector[LineNo].c_str()[0];

  while(InitString == CommentString ){
    LineNo++; 
    InitString = LineVector[LineNo].c_str()[0];
  }
  
  LineNo++;
  return LineVector[LineNo-1];
}

void ModuleInit(std::ofstream &ExportFile, std::string DetectorName, 
		std::string ModuleAddress, int GEO, int IPD, bool LastModuleFlag){
  using namespace std;
  string CommentLine = "#------------------------------------";

  ExportFile << CommentLine << endl; 
  ExportFile << "# " << DetectorName<< endl;
  ExportFile << CommentLine << endl; 
  ExportFile << QDCModel << " " << ModuleAddress << " GEO " << GEO << endl; 
  ExportFile << QDCModel << " " << ModuleAddress << " RST" << endl; 
  if (LastModuleFlag){
    ExportFile << QDCModel << " " << ModuleAddress << " IRL 1" << endl; 
    ExportFile << QDCModel << " " << ModuleAddress << " EVT 1" << endl; 
  }
  ExportFile << QDCModel << " " << ModuleAddress << " SUP NOVR ZERO LOW" << endl; 
  ExportFile << QDCModel << " " << ModuleAddress << " IPD " << IPD << endl; 
}

void THEWrite(std::ofstream &ExportFile,std::ofstream &ChkFile,
							std::string ModelNo, std::string ModuleAddress, 
							int ModuleGEO, int ModuleID,
							int THE, std::string DetectorName, std::string option,
							char* ImportFileName, std::string Position){
  using namespace std;
  
  if (ImportFileName[0] == '0'){
		if (Position == "0"){
			ExportFile << ModelNo << " " << ModuleAddress << " THE "<<
				ModuleID << " " << THE << endl;
		}else{
			ExportFile << ModelNo << " " << ModuleAddress << " THE "<<
				ModuleID << " " << "00"<< endl;
		}
	}else{
    if (option == "-d"){
      ChkFile << ModuleAddress << " " << ModuleGEO << " " << 
				ModuleID << " " << DetectorName << endl;
    }
		ExportFile << ModelNo << " " << ModuleAddress << " THE "<<
			ModuleID << " " << THE << endl;
	}
}



void TextOutput(char *ImportFileName, char *ImportSigmaFileName, char *ExportFileName,char *MapFileName, std::string option)
{
  using namespace std;

  ifstream ImportFile(ImportFileName);
  ifstream ImportSigmaFile(ImportSigmaFileName);
  ifstream MapFile(MapFileName);
  ofstream ExportFile(ExportFileName);
  int LineNo=0, ModuleNo=0;
  vector<float> ImportValue;
  vector<float> ImportSigmaValue;


  map<int, int> DetectorMap;
  map<int, int> mapExportValueUp;
  map<int, int> mapExportValueDown;
  map<int, string> MapDetectorName;
  map<int, string> MapUD;

  int ModuleIDMax;
  int ModuleNoMax;
  int forbidden;
  int DetectorNumber;
  string CtrlAddress;
  vector<string> DetectorName;
  vector<string> ModuleAddress; 
  vector<int> ModuleGEO;
  vector<int> ModuleIPD;

  float CutMargin;

  {
    vector<string> LineVector;
    
    string IniFileName="MakeVsta.ini";
    ifstream IniFile(IniFileName.c_str());
    
    if(not(IniFile)){
      cout << IniFileName << " Can not open file !! \n "<< endl;
      ExitFunction(0);
    }else{
      cout << "  IniFile = " << IniFileName <<  endl;
    }

    {
      string Line;    
      while(!IniFile.eof()){
	getline(IniFile,Line);
	LineVector.push_back(Line);
      }
    }
    
    int i=0;
    QDCModel = IniCommentString(LineVector,i);
    CutMargin = atoi(IniCommentString(LineVector,i).c_str());
    ModuleNoMax = atoi(IniCommentString(LineVector,i).c_str());
    ModuleIDMax = atoi(IniCommentString(LineVector,i).c_str());
    DetectorNumber = atoi(IniCommentString(LineVector,i).c_str());
    forbidden = atoi(IniCommentString(LineVector,i).c_str());
    CtrlAddress = IniCommentString(LineVector,i);

    if(option == "-d"){
      cout << "    " << QDCModel << "," << ModuleNoMax << "," << ModuleIDMax << "," << 
	DetectorNumber << "," << forbidden << "," << CtrlAddress <<endl ;
    }

    for (int j = 0 ; j < ModuleNoMax ; j++){
      DetectorName.push_back(IniCommentString(LineVector,i));
      if(option == "-d"){
	cout << "    " << j << "," << DetectorName[j] << endl;
      }
    }

    for (int j = 0 ; j < ModuleNoMax ; j++){
      ModuleAddress.push_back(IniCommentString(LineVector,i));
      if(option == "-d"){
	cout << "    " << j << "," << ModuleAddress[j] << endl;
      }
    }

    for (int j = 0 ; j < ModuleNoMax ; j++){
      ModuleGEO.push_back(atoi(IniCommentString(LineVector,i).c_str()));
      if(option == "-d"){
	cout << "    " << j << "," << ModuleGEO[j] << endl;
      }
    }

    for (int j = 0 ; j < ModuleNoMax ; j++){
      ModuleIPD.push_back(atoi(IniCommentString(LineVector,i).c_str()));
      if(option == "-d"){
	cout << "    " << j << "," << ModuleIPD[j] << endl;
      }
    }

  }
  
  
  if(not(MapFile)){
      cout << MapFileName << " Can not open file !! \n "<< endl;
      ExitFunction(0);
  }else{
    cout << "  MapFile = " << MapFileName <<  endl;
  }
  
  {
    int value[22];
    int mapGEOID;
    string NameValue;
    while(!MapFile.eof()){
      
      MapFile >> value[0] >> value[1] >> value[2] >> value[3] >> value[4] >> 
	value[5] >> value[6] >> value[7] >> value[8] >> value[9] >>
	value[10] >> value[11] >> value[12] >> value[13] >> value[14] >>
	value[15] >> value[16] >> value[17] >> value[18] >> value[19] >>
	value[20] >> value[21] >> NameValue ;
      
      mapGEOID = GEOIDGen(value[15],value[16]);
      DetectorMap.insert( map<int, int>::value_type( mapGEOID, value[1]) );
      MapDetectorName.insert( map<int, string>::value_type( mapGEOID, NameValue) );
      MapUD.insert( map<int, string>::value_type( mapGEOID, "U") );

      mapGEOID = GEOIDGen(value[20],value[21]);
      DetectorMap.insert( map<int, int>::value_type( mapGEOID, value[1]) );
      MapDetectorName.insert( map<int, string>::value_type( mapGEOID, NameValue) );
      MapUD.insert( map<int, string>::value_type( mapGEOID, "D") );
      
    }
  }
  

  if(ImportFileName[0] != '0'){
  
    if(not(ImportFile)){
      cout << ImportFileName << " Can not open file !! \n "<< endl;
      ExitFunction(0);
    }else{
      cout << "  " << ImportFileName << " -> "<< ExportFileName << endl;
    }
    
    {
      float value;
      while(!ImportFile.eof()){
	ImportFile >> value ;
	ImportValue.push_back(value);
      }
    }
    
    if(not(ImportSigmaFile)){
      cout << ImportSigmaFileName << " Can not open file !! \n "<< endl;
      ExitFunction(0);
    }    
    {
      float value;
      while(!ImportSigmaFile.eof()){
	ImportSigmaFile >> value ;
	ImportSigmaValue.push_back(value);
      }
    }
    

    {
      int ExValue;
      for (LineNo = 0 ; LineNo < (int)ImportValue.size() ; LineNo++){
	ExValue = ConvertFunction(ImportValue[LineNo], ImportSigmaValue[LineNo], CutMargin, forbidden);
	if (LineNo < DetectorNumber){
	  mapExportValueUp.insert( map<int, int>::value_type(LineNo+1, ExValue ) );
	}else{
	  mapExportValueDown.insert( map<int, int>::value_type(LineNo + 1 - DetectorNumber, ExValue ) );
	}
	
	if(option == "-d"){
	  cout << "    " << LineNo+1 << "," << ExValue<<  endl ;
	}
      }
      
    }
    cout << "  number of lines : " << LineNo << endl;
  }else{
      cout << "  [Zero Mode] -> "<< ExportFileName << endl;
  }


  ExportFile << "Ctrl "<< CtrlAddress << " AMSR A32" << endl; 
  {

    string ChkFileName;
    ChkFileName = string(ExportFileName);
    ChkFileName += ".chk";
    if (option == "-d"){
      cout << "  Check File Name : " << ChkFileName << endl; 
    }
    ofstream ChkFile(ChkFileName.c_str());

    int mapGEOID;
    bool LastModuleFlag = false ;
    
    for (ModuleNo = 0 ; ModuleNo < ModuleNoMax ; ModuleNo++){
      
      if(ModuleNo ==  ModuleNoMax -1){
	 LastModuleFlag = true ;
      }
      ModuleInit(ExportFile,DetectorName[ModuleNo],ModuleAddress[ModuleNo],
		 ModuleGEO[ModuleNo],ModuleIPD[ModuleNo],LastModuleFlag);
      
      for (int ModuleID = 0 ; ModuleID < ModuleIDMax ; ModuleID++){
	mapGEOID = GEOIDGen(ModuleGEO[ModuleNo],ModuleID);
	
	if( LastModuleFlag && ModuleID == ModuleIDMax - 1) {
	  ExportFile << "#   Don't Edit -->" << endl;
	  ExportFile << QDCModel << " " << ModuleAddress[ModuleNo] << " THE "<<
	    ModuleID << " 00" << endl;
	  ExportFile << "#   <-- " << endl;
	  ExitFunction(1);
	}else{
	  if (MapUD[mapGEOID] == "U"){
	    THEWrite(ExportFile, ChkFile, QDCModel, ModuleAddress[ModuleNo], ModuleGEO[ModuleNo],
		     ModuleID, mapExportValueUp[DetectorMap[mapGEOID]], MapDetectorName[mapGEOID],
							 option, ImportFileName,"U");
	  }else if(MapUD[mapGEOID] == "D"){
	    THEWrite(ExportFile, ChkFile, QDCModel, ModuleAddress[ModuleNo], ModuleGEO[ModuleNo],
		     ModuleID, mapExportValueDown[DetectorMap[mapGEOID]], MapDetectorName[mapGEOID],
		     option, ImportFileName,"D");
	  }else{
	    THEWrite(ExportFile, ChkFile, QDCModel, ModuleAddress[ModuleNo], ModuleGEO[ModuleNo],
							 ModuleID, forbidden, "None", option, ImportFileName,"0");
	  }
	}
      } 
    }
  }
}

int main(int argc, char *argv[]) 
{
  using namespace std ;

	if (argc < 4 ){
		DisplayHelp();
	}

	string option;
  if(argv[5] == NULL){
		option = "-a";
  }else{
		option = string(argv[5]);
	}


  cout << "-------------------------------" << endl; 
  cout << "MakeVsta" << endl; 

  TextOutput(argv[1],argv[2],argv[3],argv[4],option);
  ExitFunction(1);
}

